package com.menagerie.bakers.model

import androidx.compose.ui.graphics.Color
import org.jetbrains.compose.resources.DrawableResource
import bakersmenagerie.composeapp.generated.resources.Res
import bakersmenagerie.composeapp.generated.resources._10_strawberries
import kotlin.time.Duration

enum class TagType {
    COURSE,         // Breads, Apps, Desserts, Entrees, etc
    CUISINE,        // Country or Ethnicity of Origin
    FLAVOUR,        // Spicy, Sweet, Sour, Umami, Delicious
    TECHNIQUE,        // Fried, Baked, Boiled, Poached, etc
    PROTEIN,
}

//For internal use and display purposes only (see get Recipe)
enum class TTT {
    TRIED,  //We've picked up a new recipe!
    TESTED, //We've made this recipe several times
    TRUE,   //We've made this recipe a dozen times
}

data class Recipe(
    val title: String = "PLACEHOLDER",
    val description: String = "PLEASE DON'T FORGET ABOUT ME",
    val prepTime: Duration = Duration.INFINITE,
    val cookTime: Duration = Duration.INFINITE,
    val servings: String = "Enough",
    val ingredients: Map<String,Ingredient> = mapOf(),
    var instructions: List<String> = listOf(),
    val tags: Map<String, TagType> = mapOf(),
    val image: DrawableResource = Res.drawable._10_strawberries,
    val linkedRecipes : List<Recipe> = listOf(),
    var bgColor: Color = Color.White,
    var favourite: Boolean = false,
    var iMadeThis: Boolean = false,
    var ttt: TTT = TTT.TRIED

){
    /**
     * Getter for ingredients from the HashMap.
     * Primarily used to make instantiation of instructions list more readable.
     *
     * @param name key of the ingredient you want from the map
     *
     * @return a formatted string from the ingredient object with important info
     */
    fun getIngr(ingredientKey: String) : String {
        return this.ingredients[ingredientKey]!!.recipeName()
    }
}

