package com.menagerie.bakers.util

/**
 * Common safe temperature values to use where necessary.
 * Implemented for ease of unit conversion
 * Data sourced from https://www.foodsafety.gov/food-safety-charts/safe-minimum-internal-temperatures
 * unless otherwise stated
 */
enum class SafeTemps(val temp: DataWithUnit) {
    /**
     * Includes chicken, turkey, misc. poultry, and
     * all contents thereof.
     */
    POULTRY(DataWithUnit(165.0,Unit.FAHRENHEIT)),

    /**
     * Cuts of meat such as steak, flank, or roast.
     * Includes mutton, bison, and goat.
     */
    BEEF(DataWithUnit(145.0,Unit.FAHRENHEIT)),

    /**
     * All ground meat, save poultry, which is a separate category
     */
    GROUND_MEAT(DataWithUnit(160.0,Unit.FAHRENHEIT)),

    /**
     * All non-ground pork products, including raw ham.
     */
    PORK(DataWithUnit(145.0,Unit.FAHRENHEIT)),

    /**
     * Also includes venison for simplicity
     */
    RABBIT(DataWithUnit(160.0,Unit.FAHRENHEIT)),

    /**
     * All forms of fish
     */
    FISH(DataWithUnit(145.0,Unit.FAHRENHEIT)),
}