package com.menagerie.bakers.util

import androidx.compose.ui.graphics.vector.PathNode
import kotlin.math.floor

/**
 * Helper object; pair Data with Unit
 *
 * @property amount Value
 * @property unit Optional unit
 *
 * @author Blizzard Finnegan
 * @since v1.1.0
 */
data class DataWithUnit(var amount: Double = 0.0, var unit: Unit? = null){
    override fun equals(other: Any?): Boolean {
        if (other !is DataWithUnit) { return false }
        val castOther = other as DataWithUnit
        if (castOther.unit === this.unit) { return this.amount.equals(castOther.amount) }

        //Null unit check
        if (castOther.unit == null || this.unit == null) {
            return false
        }

        //Units are not equal and not null, must cast to same unit if possible
        if(IMPERIAL_UNITS.contains(castOther.unit) && IMPERIAL_UNITS.contains(this.unit)) {
           //Common imperial units. Resolve further
            when (this.unit) {
                Unit.TEASPOON -> {
                    return when (castOther.unit){
                        Unit.TABLESPOON ->
                            this.amount == (castOther.amount * ConversionFactors.TBSP_TO_TSP)
                        Unit.CUP ->
                            this.amount == (castOther.amount * ConversionFactors.CUP_TO_TSP)
                        Unit.FLUID_OUNCE ->
                            (this.amount / ConversionFactors.TBSP_TO_TSP) == (castOther.amount * ConversionFactors.FLUID_OUNCE_TO_TBSP)
                        Unit.PINT ->
                            (this.amount / ConversionFactors.CUP_TO_TSP) == (castOther.amount * ConversionFactors.PINT_TO_CUP)
                        Unit.QUART ->
                            (this.amount / ConversionFactors.CUP_TO_TSP) == (castOther.amount * ConversionFactors.QUART_TO_CUP)
                        Unit.GALLON ->
                            (this.amount / ConversionFactors.CUP_TO_TSP) == (castOther.amount * ConversionFactors.GALLON_TO_CUP)
                        else -> false
                    }
                }
                Unit.TABLESPOON -> {
                    return when (castOther.unit){
                        Unit.TEASPOON ->
                            this.amount == (castOther.amount / ConversionFactors.TBSP_TO_TSP)
                        Unit.CUP ->
                            this.amount == (castOther.amount * ConversionFactors.CUP_TO_TBSP)
                        Unit.FLUID_OUNCE ->
                            this.amount == (castOther.amount * ConversionFactors.FLUID_OUNCE_TO_TBSP)
                        Unit.PINT ->
                            (this.amount / ConversionFactors.CUP_TO_TBSP) == (castOther.amount * ConversionFactors.PINT_TO_CUP)
                        Unit.QUART ->
                            (this.amount / ConversionFactors.CUP_TO_TBSP) == (castOther.amount * ConversionFactors.QUART_TO_CUP)
                        Unit.GALLON ->
                            (this.amount / ConversionFactors.CUP_TO_TBSP) == (castOther.amount * ConversionFactors.GALLON_TO_CUP)
                        else -> false
                    }
                }
                Unit.CUP -> {
                    return when (castOther.unit){
                        Unit.TEASPOON ->
                            this.amount == (castOther.amount / ConversionFactors.CUP_TO_TSP)
                        Unit.TABLESPOON ->
                            this.amount == (castOther.amount / ConversionFactors.CUP_TO_TBSP)
                        Unit.FLUID_OUNCE ->
                            (this.amount * ConversionFactors.CUP_TO_TBSP) == (castOther.amount * ConversionFactors.FLUID_OUNCE_TO_TBSP)
                        Unit.PINT ->
                            this.amount == (castOther.amount * ConversionFactors.PINT_TO_CUP)
                        Unit.QUART ->
                            this.amount == (castOther.amount * ConversionFactors.QUART_TO_CUP)
                        Unit.GALLON ->
                            this.amount == (castOther.amount * ConversionFactors.GALLON_TO_CUP)
                        else -> false
                    }
                }
                Unit.FLUID_OUNCE -> {
                    return when (castOther.unit){
                        Unit.TEASPOON ->
                            (this.amount * ConversionFactors.FLUID_OUNCE_TO_TBSP) == (castOther.amount * ConversionFactors.TBSP_TO_TSP)
                        Unit.TABLESPOON ->
                            (this.amount * ConversionFactors.FLUID_OUNCE_TO_TBSP) == castOther.amount
                        Unit.CUP ->
                            (this.amount * ConversionFactors.FLUID_OUNCE_TO_TBSP) == (castOther.amount * ConversionFactors.CUP_TO_TBSP)
                        Unit.PINT ->
                            (this.amount * ConversionFactors.FLUID_OUNCE_TO_TBSP)== (castOther.amount * ConversionFactors.PINT_TO_CUP * ConversionFactors.CUP_TO_TBSP)
                        Unit.QUART ->
                            (this.amount * ConversionFactors.FLUID_OUNCE_TO_TBSP)== (castOther.amount * ConversionFactors.QUART_TO_CUP * ConversionFactors.CUP_TO_TBSP)
                        Unit.GALLON ->
                            (this.amount * ConversionFactors.FLUID_OUNCE_TO_TBSP)== (castOther.amount * ConversionFactors.GALLON_TO_CUP * ConversionFactors.CUP_TO_TBSP)
                        else -> false
                    }
                }
                Unit.PINT -> {
                    return when (castOther.unit){
                        Unit.TEASPOON ->
                            (this.amount * ConversionFactors.PINT_TO_CUP) == (castOther.amount / ConversionFactors.CUP_TO_TSP)
                        Unit.TABLESPOON ->
                            (this.amount * ConversionFactors.PINT_TO_CUP) == (castOther.amount / ConversionFactors.CUP_TO_TBSP)
                        Unit.FLUID_OUNCE ->
                            (this.amount * ConversionFactors.PINT_TO_CUP * ConversionFactors.CUP_TO_TBSP) == (castOther.amount * ConversionFactors.FLUID_OUNCE_TO_TBSP)
                        Unit.CUP ->
                            (this.amount * ConversionFactors.PINT_TO_CUP)== castOther.amount
                        Unit.QUART ->
                            (this.amount * ConversionFactors.PINT_TO_CUP)== (castOther.amount * ConversionFactors.QUART_TO_CUP)
                        Unit.GALLON ->
                            (this.amount * ConversionFactors.PINT_TO_CUP)== (castOther.amount * ConversionFactors.GALLON_TO_CUP)
                        else -> false
                    }
                }
                Unit.QUART -> {
                    return when (castOther.unit){
                        Unit.TEASPOON ->
                            (this.amount * ConversionFactors.QUART_TO_CUP) == (castOther.amount / ConversionFactors.CUP_TO_TSP)
                        Unit.TABLESPOON ->
                            (this.amount * ConversionFactors.QUART_TO_CUP) == (castOther.amount / ConversionFactors.CUP_TO_TBSP)
                        Unit.FLUID_OUNCE ->
                            (this.amount * ConversionFactors.QUART_TO_CUP) == (castOther.amount * ConversionFactors.FLUID_OUNCE_TO_TBSP / ConversionFactors.CUP_TO_TBSP)
                        Unit.CUP ->
                            (this.amount * ConversionFactors.QUART_TO_CUP)== castOther.amount
                        Unit.PINT ->
                            (this.amount * ConversionFactors.QUART_TO_CUP)== (castOther.amount * ConversionFactors.PINT_TO_CUP)
                        Unit.GALLON ->
                            (this.amount * ConversionFactors.QUART_TO_CUP)== (castOther.amount * ConversionFactors.GALLON_TO_CUP)
                        else -> false
                    }
                }
                Unit.GALLON -> {
                    return when (castOther.unit){
                        Unit.TEASPOON ->
                            (this.amount * ConversionFactors.GALLON_TO_CUP) == (castOther.amount * ConversionFactors.TBSP_TO_TSP)
                        Unit.TABLESPOON ->
                            (this.amount * ConversionFactors.GALLON_TO_CUP) == (castOther.amount / ConversionFactors.CUP_TO_TBSP)
                        Unit.FLUID_OUNCE ->
                            (this.amount * ConversionFactors.GALLON_TO_CUP) == (castOther.amount * ConversionFactors.FLUID_OUNCE_TO_TBSP / ConversionFactors.CUP_TO_TBSP)
                        Unit.CUP ->
                            (this.amount * ConversionFactors.GALLON_TO_CUP)== castOther.amount
                        Unit.PINT ->
                            (this.amount * ConversionFactors.GALLON_TO_CUP)== (castOther.amount * ConversionFactors.PINT_TO_CUP * ConversionFactors.CUP_TO_TBSP)
                        Unit.QUART ->
                            (this.amount * ConversionFactors.GALLON_TO_CUP)== (castOther.amount * ConversionFactors.QUART_TO_CUP * ConversionFactors.CUP_TO_TBSP)
                        else -> false
                    }
                }
                Unit.POUND -> {
                    if (castOther.unit == Unit.OUNCE)
                        return castOther.amount == (this.amount * ConversionFactors.LB_TO_OZ)
                    return false
                }
                Unit.OUNCE -> {
                    if (castOther.unit == Unit.POUND) {
                        return this.amount == (castOther.amount * ConversionFactors.LB_TO_OZ)
                    }
                    return false
                }
                Unit.FAHRENHEIT -> return false
                else -> return false
            }
        }
        else if (METRIC_UNITS.contains(castOther.unit) && METRIC_UNITS.contains(this.unit)) {
            return if (this.unit == Unit.KILOGRAM && castOther.unit == Unit.GRAM){
                this.amount == castOther.amount * ConversionFactors.BASE_TO_KILO
            }
            else if (this.unit == Unit.GRAM && castOther.unit == Unit.KILOGRAM){
                this.amount * ConversionFactors.BASE_TO_KILO == castOther.amount
            }
            else if (this.unit == Unit.MILLILITER && castOther.unit == Unit.LITER){
                this.amount * ConversionFactors.MILLI_TO_BASE == castOther.amount
            }
            else if (this.unit == Unit.LITER && castOther.unit == Unit.MILLILITER){
                this.amount == castOther.amount * ConversionFactors.MILLI_TO_BASE
            }
            else false
        } else {
            //Both units are not in the same group of units, try again with converted type
            return castOther.imperialMetricConvert() == this
        }
    }
    override fun toString(): String {
        val printAmount: String

        if (this.amount == -1.0) return ""

        if (this.amount  % 1.0 == 0.0) {
            //If the number is a whole number, just print the whole number
            printAmount = floor(this.amount).toInt().toString()
        } else if (METRIC_UNITS.contains(this.unit)) {
            //Allow decimals for metric values
            //TODO: Format decimals reasonably, so that fully extended decimals are not shown.
            // Consider regex for this purpose, if necessary
            printAmount = this.amount.toString()
        } else if ((this.amount * 2) % 1.0 == 0.0) {
            //If multiplying by 2 is a whole number
            //Since the previous clause parses whole numbers already, this will only resolve on
            //values that contain a 1/2
            if (this.amount > 1)
                printAmount = floor(this.amount).toInt().toString().plus(SpecialCharacters.ONE_HALF.toString())
            else
                printAmount = SpecialCharacters.ONE_HALF.toString()
        } else if ((this.amount * 3) % 1.0 == 0.0) {
            //If multiplying by 3 is a whole number
            //Since the previous clause parses whole numbers already, this will only resolve on
            //values that contain a 1/3
            // Compare against 1/2
            if (floor(this.amount) > 1) {
                if (floor(this.amount) + 0.5  == this.amount)
                    printAmount = floor(this.amount).toInt().toString().plus(SpecialCharacters.TWO_THIRD)
                else
                    printAmount = floor(this.amount).toInt().toString().plus(SpecialCharacters.ONE_THIRD)
            } else {
                if (floor(this.amount * 3).toInt() == 1)
                    printAmount = SpecialCharacters.ONE_THIRD.toString()
                else
                    printAmount = SpecialCharacters.TWO_THIRD.toString()
            }
        } else if ((this.amount * 4) % 1.0 == 0.0) {
            //If multiplying by 4 is a whole number
            //Since 1/2 has already been removed, this will resolve on values that contain 1/4 or 3/4
            // Compare against 1/2
            if(floor(this.amount) + 0.5 > this.amount)
                printAmount = SpecialCharacters.ONE_QUARTER.toString()
            else
                printAmount = SpecialCharacters.THREE_QUARTER.toString()
        } else {
            printAmount = this.amount.toString()
        }
        if (this.unit == null) return printAmount


        return printAmount.plus(this.unit?.abbreviation)
    }
    override fun hashCode(): Int {
        return this.amount.hashCode() * 31 + this.unit.hashCode()
    }

    fun debugPrint(): String{
        return "DataWithUnit[amount: ".
        plus(this.amount).
        plus("; Unit: ").
        plus(this.unit?.abbreviation).
        plus("]")
    }
    /**
     * Simplify data to the most readable unit. Value is reduced in toString override
     * @return copy of this object with the reduced units
     */
    fun simplifyUnit() : DataWithUnit {
        //TODO: Implement
        //TODO: Determine if value contains fraction, act accordingly?
        when (this.unit) {
            null -> return this
            Unit.TEASPOON -> {
                /*
                Fractions acceptable
                Compatible:
                Tablespoon, cup

                If num > 48, convert to cup
                else if num > 3 and num % 3 = 0, convert to tbsp
               */
            }
            Unit.TABLESPOON -> {
                /*
                fractions acceptable
                Compatible:
                Teaspoon, cup
                if num < 1, convert to teaspoon
                if num > 4 and num % 4 = 0, convert to cup
                 */
            }
            Unit.CUP -> {
                /*
                fractions acceptable
                Compatible:
                Tablespoon, Teaspoon
                if num < 1, always convert to tablespoon; recurse
                 */
            }
            Unit.FLUID_OUNCE -> {
                /*
                fractions don't make sense
                Compatible:
                Cup, Teaspoon, Tablespoon
                convert to teaspoon, recurse
                 */
            }
            Unit.PINT -> {
                /*
                fractions don't make sense
                Compatible:
                cup, quart, gallon
                if num < 1, convert to quart
                If num % 4, convert to gallon
                if num % 2, convert to quart
                 */
            }
            Unit.QUART -> {
                /*
                fractions don't make sense
                compatible:
                cup, pint, gallon
                if num * 2 < 1, convert to cup
                if num < 1, convert to pint
                 */
            }
            Unit.GALLON -> {
                /*
                fractions don't make sense
                compatible:
                cup, pint, quart
                if num * 4 < 1, convert to cup, recurse
                if num * 2 < 1, convert to pint
                if num < 1, convert to quart
                 */
            }
            Unit.KILOGRAM -> {
                /*
                Fractions don't make sense, decimals ok
                compatible:
                gram
                 */
            }
            Unit.GRAM -> {
                /*
                Fractions don't make sense, decimals ok
                compatible:
                kilogram
                 */
            }
            Unit.MILLILITER -> {
                /*
                Fractions don't make sense, decimals ok
                compatible:
                liter
                 */
            }
            Unit.LITER -> {
                /*
                Fractions don't make sense, decimals ok
                compatible:
                milliliter
                 */
            }
            Unit.POUND -> {
                /*
                Fractions don't make sense, use lb/oz for print statement
                compatible:
                ounce
                 */
            }
            Unit.OUNCE -> {
                /*
                fractions don't make sense; decimals ok, consider rounding off
                compatible:
                pound
                 */
            }
            else -> {}
        }
        return this
    }
    /**
     * Convert the given value from imperial to metric, or vice-versa
     *
     * @return copy of this object converted to most useful unit in the opposite unit type
     */
    fun imperialMetricConvert() : DataWithUnit {
        TODO()
        //simplifyUnit(generatedObject)
    }
    /**
     * Helper function for temperature conversion.
     *
     * @return Copy of this object with converted temp if valid input, null otherwise
     */
    fun tempConvert(): DataWithUnit?{
        when (this.unit) {
            Unit.FAHRENHEIT -> {
                val convertedTemp = ( this.amount - F_TO_C_CONVERT_OFFSET ) * F_TO_C_CONVERT_RATIO
                return DataWithUnit(convertedTemp,Unit.CELSIUS)
            }
            Unit.CELSIUS -> {
                val convertedTemp = (this.amount / F_TO_C_CONVERT_RATIO) + F_TO_C_CONVERT_OFFSET
                return DataWithUnit(convertedTemp,Unit.FAHRENHEIT)
            }
            else -> {}
        }
        return null
    }

    fun scale(factor: Double): DataWithUnit{
        val returnVar = DataWithUnit(this.amount*factor, this.unit)
        returnVar.simplifyUnit()
        return returnVar
    }


}



/**
 * Map of imperial units, and their most reasonable metric conversion
 */
val IMPERIAL_TO_METRIC:HashMap<Unit, Unit> = hashMapOf(
    Unit.TEASPOON to Unit.MILLILITER,
    Unit.TABLESPOON to Unit.MILLILITER,
    Unit.CUP to Unit.MILLILITER,
    Unit.FLUID_OUNCE to Unit.MILLILITER,
    Unit.PINT to Unit.LITER,
    Unit.QUART to Unit.LITER,
    Unit.GALLON to Unit.LITER,
    Unit.POUND to Unit.KILOGRAM,
    Unit.OUNCE to Unit.GRAM,
)

/**
 * Map of metric units, and their most reasonable metric conversions
 * Due to significantly less units in metric, if this map is used, calling
 * [DataWithUnit.simplifyUnit] is advised
 */
val METRIC_TO_IMPERIAL:HashMap<Unit, Unit> = hashMapOf(
    Unit.MILLILITER to Unit.TEASPOON,
    Unit.LITER to Unit.PINT,
    Unit.KILOGRAM to Unit.POUND,
    Unit.GRAM to Unit.OUNCE
)

/**
 * Array of units that are explicitly metric
 */
val METRIC_UNITS:Array<Unit> = arrayOf(
    Unit.KILOGRAM,
    Unit.GRAM,
    Unit.MILLILITER,
    Unit.LITER,
    Unit.CELSIUS
)

/**
 * Array of units that are explicitly imperial
 */
val IMPERIAL_UNITS:Array<Unit> = arrayOf(
    Unit.TEASPOON,
    Unit.TABLESPOON,
    Unit.CUP,
    Unit.FLUID_OUNCE,
    Unit.PINT,
    Unit.QUART,
    Unit.GALLON,
    Unit.POUND,
    Unit.OUNCE,
    Unit.FAHRENHEIT
)

/*
Conversions:

1 tbsp -> 3 tsp
4 tbsp -> 1/4 cup; 1tbsp -> 1/16 cup
16 teaspoon -> 1/3 cup; 1tsp -> 1/48 cup
1 tsp -> 4.928922mL
1fl oz -> 2 tbsp
2 cup -> 1 pint
4 cup -> 1 quart
16 cup -> 1 gallon

1 lb -> 0.45359237 kg
1 lb -> 16oz
 */

/**
 * Storage of conversion factors for unit conversion
 *
 * All Values stored within are multiplicative from the larger unit to the smaller
 * i.e. TBSP_TO_TSP is used as tablespoon * TBSP_TO_TSP = teaspoon
 */
private data object ConversionFactors {
    const val TBSP_TO_TSP = 3.0 as Double
    const val CUP_TO_TBSP = 16.0 as Double
    const val CUP_TO_TSP = 48.0 as Double
    const val TSP_TO_ML = 4.928922 as Double
    const val FLUID_OUNCE_TO_TBSP = 2.0 as Double
    const val PINT_TO_CUP = 2.0 as Double
    const val QUART_TO_CUP = 4.0 as Double
    const val GALLON_TO_CUP = 16.0 as Double
    const val LB_TO_OZ = 16.0 as Double
    const val KG_TO_OZ = 35.27396195 as Double
    const val BASE_TO_KILO = 1000.0 as Double
    const val MILLI_TO_BASE = 1000.0 as Double
}

/**
 * Unicode codepoints used for displaying fractions. Stored here for general use
 */
data object SpecialCharacters {
    const val ONE_QUARTER = '\u00bc'
    const val ONE_THIRD = '\u2153'
    const val ONE_HALF = '\u00bd'
    const val TWO_THIRD = '\u2154'
    const val THREE_QUARTER = '\u00be'
}

const val F_TO_C_CONVERT_RATIO = 9.0 / 5.0
const val F_TO_C_CONVERT_OFFSET = 32

/**
 * Commonly used units in cooking. New units added here should also add the relevant data to
 * the [ConversionFactors], [METRIC_TO_IMPERIAL], [METRIC_UNITS], [IMPERIAL_TO_METRIC], and
 * [IMPERIAL_UNITS] as appropriate. Update to [DataWithUnit.equals] may also be necessary
 *
 * Enum also includes printable string with appropriate abbreviation.
 */
enum class Unit (val abbreviation: String){
    /**
     * Teaspoon
     *
     * Some fractions acceptable in this unit: quarter, third, half.
     * Smaller values should be either left as decimal(?) or converted to [MILLILITER] (?)
     */
    TEASPOON("tsp"),

    /**
     * Tablespoon
     *
     * Some fractions acceptable in this unit: quarter, third, half.
     * Smaller values should be converted to [TEASPOON]
     */
    TABLESPOON ("tbsp"),

    /**
     * Cup
     *
     * Some fractions acceptable for this unit: quarter, third, half
     * smaller values should be converted to [TABLESPOON] or [TEASPOON]
     */
    CUP ("c"),

    /**
     * Fluid ounce
     *
     * Smallest liquid volumetric value. Whole numbers only.
     * Store decimals for precision, do not print.
     */
    FLUID_OUNCE ("fl. oz."),

    /**
     * Pint
     *
     * Whole numbers only.
     * Smaller values should be down-converted to [CUP], at minimum.
     */
    PINT("pint"),  //Whole numbers only. If fraction, consider down-converting

    /**
     * Quart
     *
     * Whole numbers only.
     * Smaller values should be down-converted to [CUP], at minimum.
     */
    QUART("qt."), //whole numbers only. If fraction, consider down-converting

    /**
     * Gallon
     *
     * Whole numbers only.
     * Smaller values should be down-converted to [CUP], at minimum.
     * Possible down-conversion to Quart or Pint, depending on implementation.
     */
    GALLON("gal."), //whole numbers only. If fraction, consider down-converting

    /**
     * Kilogram
     *
     * No fractions; decimals acceptable.
     * Values less than 1 should be converted down to [GRAM].
     */
    KILOGRAM("kg"), //decimals only

    /**
     * Gram
     *
     * No fractions; Values less than 1 are questionably useful.
     * Values larger than 1000 should be converted up to [KILOGRAM].
     */
    GRAM("g"), //decimals only

    /**
     * Milliliter
     *
     * No fractions; values less than 1 are questionably useful.
     * Values larger than 1000 should be converted up to [LITER].
     */
    MILLILITER("mL"), //decimals only

    /**
     * Liter
     *
     * No fractions; decimals acceptable.
     * Values smaller than 1 should be down-converted to [MILLILITER]
     */
    LITER("L"), //decimals only

    /**
     * Pound
     *
     * No fractions, no decimals.
     * UNIQUE PRINT FORMATTING: lb, decimal ounces
     *
     * Values less than 1 should down-convert to [OUNCE]
     */
    POUND("lb"), // lb oz formatting

    /**
     * Ounce (weight)
     *
     * No fractions, decimals acceptable.
     * Values less than 0.1 are questionably useful.
     */
    OUNCE("oz."), //Decimals only

    /**
     * Degrees Celsius
     *
     * No fractions, decimals encouraged for conversion to [FAHRENHEIT] compatibility.
     */
    CELSIUS("\u00b0C"), //Decimals only

    /**
     * Degrees Fahrenheit
     *
     * No fractions, decimals are questionably useful.
     */
    FAHRENHEIT("\u00b0F") //Decimals only
}

