package com.menagerie.bakers.view

import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.grid.GridCells
import androidx.compose.foundation.lazy.grid.LazyVerticalGrid
import androidx.compose.foundation.lazy.grid.rememberLazyGridState
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.ArrowDropDown
import androidx.compose.material3.Button
import androidx.compose.material3.Icon
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import com.menagerie.bakers.DisplayManager
import com.menagerie.bakers.Orientation
import com.menagerie.bakers.Size
import com.menagerie.bakers.model.TagType


@Composable
fun BookShelf(
    onClick: (String) -> Unit,
    tags: Map<String, TagType>,
) {
    Box(
        modifier = Modifier
    ) {

        Column(
            modifier = Modifier
                .padding(top = 10.dp, start = 20.dp, end = 20.dp)
                .align(Alignment.Center),
            horizontalAlignment = Alignment.CenterHorizontally,
            verticalArrangement = Arrangement.Center,
        ) {

            Spacer(modifier = Modifier.weight(when(DisplayManager.size){
                Size.Small -> .05f
                Size.Medium -> .1f
                Size.Large -> .5f
            }))

            Text(text = "What Do You Want To Cook Today?", textAlign = TextAlign.Center)

            Row(modifier = Modifier.weight(.25f)) {
                Button( modifier = Modifier.padding(start = 3.dp, end = 3.dp),
                    onClick = { onClick("") }) {
                    Text("Everything!")
                }
                Button( modifier = Modifier.padding(start = 3.dp, end = 3.dp),
                    onClick = {onClick("SURPRISE")}) {
                    Text("Anything!")
                }
            }
            Spacer(modifier = Modifier.weight(when(DisplayManager.size){
                Size.Small -> .05f
                Size.Medium -> if(DisplayManager.orientation == Orientation.Portrait) .1f else .05f
                Size.Large -> .5f
            } ))

            for (type in tags.values.toSet().toList().sorted()) // get all unique tag types
            {
                var show by remember { mutableStateOf(DisplayManager.size == Size.Large) }

                Box(modifier = Modifier.weight(if(show) 1f else .1f)) {
                    Column {
                        var list =
                            tags.filterValues { it == type }.keys.toList() // get list of all keys that have this value
                        list = list.sorted()

                        when(DisplayManager.size) {
                            Size.Small, Size.Medium ->
                            Row {
                                Text(
                                    "Cook By ".plus(type.name),
                                    modifier = Modifier.clickable {
                                        show = !show
                                    })
                                Icon(imageVector = Icons.Default.ArrowDropDown, "Open")
                            }
                            else -> {
                                Text("Cook By ".plus(type.name))
                            }
                        }

                        val listState = rememberLazyGridState()

                        if(show) {
                            LazyVerticalGrid(
                                columns = GridCells.Adaptive(minSize = 140.dp),
                                state = listState,
                                modifier = Modifier.fillMaxWidth()
                            ) {
                                items(list.size) { item ->
                                    val tag = list[item]
                                    Button(
                                        onClick = { onClick(tag) },
                                        modifier = Modifier.padding(start = 3.dp, end = 3.dp)
                                    ) {
                                        Text(tag)
                                    }
                                }
                            }
                        }
                    }
                }
            }
            Spacer(modifier = Modifier.weight(.0625f))
        }
    }
}