package com.menagerie.bakers.view


import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.material.MaterialTheme
import androidx.compose.material3.Button
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import com.menagerie.bakers.Device
import com.menagerie.bakers.DisplayManager
import com.menagerie.bakers.Size

@Composable
fun HomeScreen(
    onGo: () -> Unit,
    onSurprise: () -> Unit,
    onClose: () -> Unit,
) {
    val text = "Recipes"
    HeaderText(text, onGo, onSurprise, onClose)
}

@Composable
fun HeaderText(
    text: String,
    onGo: () -> Unit,
    onSurprise: () -> Unit,
    onClose: () -> Unit,
) {

    val header = when(DisplayManager.size){
        Size.Small -> MaterialTheme.typography.h2
        Size.Medium -> MaterialTheme.typography.h2
        Size.Large -> MaterialTheme.typography.h1
    }

    val lowHead = when(DisplayManager.size) {
        Size.Small -> MaterialTheme.typography.h4
        Size.Medium -> MaterialTheme.typography.h4
        Size.Large -> MaterialTheme.typography.h3
    }

    Box(
        modifier = Modifier
            .fillMaxSize()
    ) {
        Spacer(modifier = Modifier.fillMaxWidth())
        Column(
            modifier = Modifier
                .fillMaxSize()
                .padding(top = 10.dp)
                .align(Alignment.Center),
            horizontalAlignment = Alignment.CenterHorizontally,
            verticalArrangement = Arrangement.Center,
        ) {
            Spacer(modifier = Modifier.weight(1f))
            Text (
                text = "Baker's Menagerie",
                style = header,
                textAlign = TextAlign.Center
            )
           Text (
                text = "An Allbright Family Cookbook",
                style = lowHead,
                textAlign = TextAlign.Center
            )
            Spacer(modifier = Modifier.weight(1f))
           Button(
                modifier = Modifier.padding(12.dp),
                onClick = onGo
            ) {
                Text(text = text)
            }
            if(DisplayManager.device == Device.Web)
            {
                Button(
                    modifier = Modifier.padding(12.dp),
                    onClick = onSurprise
                ){
                    Text("Surprise Me!")
                }
            }
            if(DisplayManager.device == Device.Desktop)
            {
                Button(modifier = Modifier.padding(12.dp),
                    onClick = onClose
                ) {
                    Text("Close")
                }
            }
        }
    }
}