package com.menagerie.bakers.view.details


import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.border
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.selection.SelectionContainer
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.rotate
import androidx.compose.ui.draw.shadow
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.ColorFilter
import androidx.compose.ui.graphics.luminance
import androidx.compose.ui.unit.dp
import bakersmenagerie.composeapp.generated.resources.Res
import bakersmenagerie.composeapp.generated.resources.chef
import com.menagerie.bakers.DisplayManager
import com.menagerie.bakers.Size
import com.menagerie.bakers.model.Recipe
import com.menagerie.bakers.util.rememberClipboardController
import org.jetbrains.compose.resources.painterResource


@Composable
fun IngredientItem(recipe: Recipe, ingredient: String, index: Int) {
    Box(modifier = Modifier.padding(start = 16.dp, end = 16.dp, top = 8.dp)) {
        Box(
            modifier = Modifier
                .fillMaxWidth()
                .padding(top = 8.dp)
                .border(
                    width = 2.dp,
                    color = if (recipe.bgColor != MaterialTheme.colorScheme.primaryContainer) {
                        if (index % 2 == 0) {
                            MaterialTheme.colorScheme.secondaryContainer
                        } else {
                            MaterialTheme.colorScheme.onBackground
                        }
                    } else
                        if (index % 2 == 0) {
                            MaterialTheme.colorScheme.background
                        } else {
                            MaterialTheme.colorScheme.onBackground
                        },
                    shape = RoundedCornerShape(35.dp)
                )
        ) {
            SelectionContainer {

                val style = when (DisplayManager.size) {
                    Size.Small -> MaterialTheme.typography.bodySmall
                    Size.Medium -> MaterialTheme.typography.bodyMedium
                    Size.Large -> MaterialTheme.typography.bodyLarge
                }

                Text(
                    text = ingredient,
                    style = style,
                    modifier = Modifier
                        .fillMaxWidth()
                        .padding(start = 55.dp, end = 8.dp, top = 16.dp, bottom = 16.dp),
                )
            }
        }

        Box(
            modifier = Modifier.padding(start = 4.dp)
        ) {
            Box(
                modifier = Modifier
                    .size(45.dp)
                    .shadow(elevation = 10.dp, shape = CircleShape)
                    .background(
                        if (recipe.bgColor != MaterialTheme.colorScheme.primaryContainer) MaterialTheme.colorScheme.secondaryContainer else MaterialTheme.colorScheme.background,
                        CircleShape
                    ),
            ) {

                val clipboardController = rememberClipboardController()

                IconButton(onClick = {
                    val actualIngredient =
                        ingredient.split(' ', ',', limit = 3).getOrElse(2) { ingredient }
                    clipboardController.copyToClipboard(actualIngredient)

                }) {
                    Image(
                        painter = painterResource(Res.drawable.chef),
                        contentDescription = null,
                        modifier = Modifier.padding(12.dp).rotate(-30f),
                        colorFilter = ColorFilter.tint(if (recipe.bgColor.luminance() > 0.3) Color.Companion.Black else Color.White)
                    )
                }

            }
        }
    }
}
