package com.menagerie.bakers.view.details

import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.aspectRatio
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.material3.Button
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableFloatStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.input.nestedscroll.NestedScrollConnection
import androidx.compose.ui.input.nestedscroll.NestedScrollSource
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.unit.Velocity
import androidx.compose.ui.unit.dp
import androidx.navigation.NavHostController
import com.menagerie.bakers.DisplayManager
import com.menagerie.bakers.RecipeAppScreen
import com.menagerie.bakers.Size
import com.menagerie.bakers.model.Recipe
import org.jetbrains.compose.resources.painterResource
import com.menagerie.bakers.view.util.RecipeScaleSlider
import com.menagerie.bakers.view.util.BackForwardButton


@Composable
fun RecipeDetailsLarge(
    recipe: Recipe,
    goForward: () -> Unit,
    multipleRecipes: Boolean,
    animate: Boolean,
    navController: NavHostController,
    navTo: String,
    discreet: Boolean
) {
    val nestedScrollConnection = remember {
        object : NestedScrollConnection {
            override fun onPreScroll(
                available: Offset, source: NestedScrollSource
            ): Offset {
                return Offset.Zero
            }

            override suspend fun onPreFling(available: Velocity): Velocity {
                return super.onPreFling(available)
            }
        }
    }

    Box(
        modifier = Modifier.fillMaxSize()
            .background(if (recipe.bgColor == MaterialTheme.colorScheme.primaryContainer) MaterialTheme.colorScheme.secondaryContainer else MaterialTheme.colorScheme.background)
    ) {
        var reduce by remember { mutableStateOf(true) }
        var slider by remember { mutableFloatStateOf(1f) }

        Row {
            Box(
                modifier = Modifier.fillMaxSize()
                    .background(color = if (recipe.bgColor == MaterialTheme.colorScheme.primaryContainer) MaterialTheme.colorScheme.secondaryContainer else MaterialTheme.colorScheme.background)
                    .weight(1f)
            ) {
                val ingredientState = rememberLazyListState()
                val stepsState = rememberLazyListState()

                Box(
                    modifier = Modifier.fillMaxSize()
                ) {
                    Column(
                        modifier = Modifier.padding(top = 64.dp)
                    ) {
                        recipeHeader(recipe, slider)

                        if (recipe.linkedRecipes.isNotEmpty()) {
                            Column(modifier = Modifier.padding(all = 16.dp)) {
                                Text(text = "Recipes Referenced in this Recipe:")

                                Row {
                                    for (link in recipe.linkedRecipes) {
                                        Button(onClick = {
                                            navController.navigate(
                                                RecipeAppScreen.Details.name.plus(
                                                    "/${link.title}"
                                                )
                                            )
                                        }) {
                                            Text(text = link.title)
                                        }
                                    }
                                }
                            }
                        }

                        RecipeScaleSlider(
                            sliderPosition = slider,
                            steps = if (reduce) 2 else 5,
                            rangeEnd = if (reduce) 1f else 4f,
                            rangeStart = if (reduce) .25f else 1f,
                            reduce = reduce,
                            onClick = {
                                reduce = !reduce
                                slider = 1f
                            }
                        ) { slider = it }

                        Row {

                            if (DisplayManager.size == Size.Large) {
                                Column(modifier = Modifier.weight(.5f)) {
                                    Box(modifier = Modifier.padding(32.dp)) {
                                        Image(
                                            painter = painterResource(recipe.image),
                                            contentDescription = null,
                                            modifier = Modifier.aspectRatio(1f)
                                                .align(Alignment.Center)
                                                .padding(16.dp)
                                                .size(400.dp).clip(CircleShape)
                                        )
                                    }
                                }
                            }

                            Column(modifier = Modifier.weight(.5f)) {
                                ingredientsHeader(recipe, animate)
                                LazyColumn(
                                    contentPadding = PaddingValues(64.dp),
                                    userScrollEnabled = true,
                                    verticalArrangement = Arrangement.Absolute.spacedBy(16.dp),
                                    modifier = Modifier.nestedScroll(nestedScrollConnection),
                                    state = ingredientState
                                ) {
                                    ingredients(recipe, slider, animate)
                                }
                            }
                            Column(modifier = Modifier.weight(1f)) {
                                stepsHeader(recipe, animate)
                                LazyColumn(
                                    contentPadding = PaddingValues(64.dp),
                                    userScrollEnabled = true,
                                    verticalArrangement = Arrangement.Absolute.spacedBy(16.dp),
                                    modifier = Modifier.nestedScroll(nestedScrollConnection),
                                    state = stepsState
                                ) {
                                    steps(recipe, slider, animate, discreet)
                                }
                            }
                        }
                    }
                }
            }
        }

        Row {
            BackForwardButton(
                {
                    navController.navigate(navTo) {
                        popUpTo(navTo) { inclusive = true }
                    }
                },
                text = if (navTo == RecipeAppScreen.History.name) "Back to History " else "Back to Recipes "
            )
            Spacer(modifier = Modifier.weight(1f))

            if (multipleRecipes)
                BackForwardButton(goForward, true, " Next")
        }
    }
}


