package com.menagerie.bakers.view.list


import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.WindowInsets
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.systemBars
import androidx.compose.foundation.layout.windowInsetsPadding
import androidx.compose.foundation.lazy.grid.GridCells
import androidx.compose.foundation.lazy.grid.LazyGridState
import androidx.compose.foundation.lazy.grid.LazyVerticalGrid
import androidx.compose.foundation.lazy.grid.rememberLazyGridState
import androidx.compose.material3.Card
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.derivedStateOf
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import com.menagerie.bakers.Device
import com.menagerie.bakers.DisplayManager
import com.menagerie.bakers.Orientation
import com.menagerie.bakers.Size
import com.menagerie.bakers.model.Recipe

@Composable
fun RecipesListScreen(
    items: List<Recipe>,
    onClick: (recipe: Recipe) -> Unit,
    animate: Boolean,
) {
    Box(
        modifier = Modifier.fillMaxSize()
    ) {
        val listState = rememberLazyGridState()
        if (items.isEmpty()) {
            Card { Text("NO RECIPE FOUND") }
        }
        else {

            LazyVerticalGrid(
                state = listState, columns = GridCells.Adaptive(minSize = 400.dp)
            ) {
                if (DisplayManager.device == Device.Android)
                    item {
                        Spacer(modifier = Modifier.windowInsetsPadding(WindowInsets.systemBars))
                    }
                items(items.size) { item ->
                    val recipe = items[item]
                    recipe.bgColor = (if(item % 2 == 0) MaterialTheme.colorScheme.primaryContainer else MaterialTheme.colorScheme.secondaryContainer)
                    RecipeListItemWrapper(
                        scrollDirection = listState.isScrollingUp(),
                        animate = animate,
                        child = {
                            RecipeListItem(
                                recipe = recipe,
                                animate = animate,
                                onClick = onClick,
                                largeList = items.size > when(DisplayManager.size) {

                                    Size.Small -> 20

                                    Size.Medium ->
                                        if (DisplayManager.orientation == Orientation.Portrait) 20
                                        else 50

                                    Size.Large ->
                                        if (DisplayManager.orientation == Orientation.Portrait) 30
                                        else 70
                                }
                            )
                        }
                    )
                }
            }
        }
    }
}

@Composable
private fun LazyGridState.isScrollingUp(): Boolean {
    var previousIndex by remember(this) { mutableStateOf(firstVisibleItemIndex) }
    var previousScrollOffset by remember(this) { mutableStateOf(firstVisibleItemScrollOffset) }
    return remember(this) {
        derivedStateOf {
            if (previousIndex != firstVisibleItemIndex) {
                previousIndex > firstVisibleItemIndex
            } else {
                previousScrollOffset >= firstVisibleItemScrollOffset
            }.also {
                previousIndex = firstVisibleItemIndex
                previousScrollOffset = firstVisibleItemScrollOffset
            }
        }
    }.value
}