package com.menagerie.bakers.view.util

import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.automirrored.filled.ArrowForward
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.unit.dp
import com.menagerie.bakers.DisplayManager
import com.menagerie.bakers.Orientation
import com.menagerie.bakers.Size


@Composable
fun BackForwardButton(
    go : () -> Unit,
    forward : Boolean = false,
    text: String,
) {

    val start = when (DisplayManager.size) {
        Size.Small -> 12.dp
        Size.Medium -> 22.dp
        Size.Large -> 32.dp
    }
    val top = when (DisplayManager.size) {
        Size.Small -> 8.dp
        Size.Medium -> 12.dp
        Size.Large -> 16.dp
    }
    val end = when (DisplayManager.size) {
        Size.Small -> 12.dp
        Size.Medium -> 22.dp
        Size.Large -> 32.dp
    }

    Box(
        modifier = Modifier.padding(
            start = start,
            top = top,
            end = end,
        ).clip(
            RoundedCornerShape(50)
        ).clickable {
            go()
        }.background(
            color = Color.Black, shape = RoundedCornerShape(50)
        )
    ) {
        Row(verticalAlignment = Alignment.CenterVertically) {

            if(forward.not())
                Icon(
                    imageVector = Icons.AutoMirrored.Default.ArrowBack,
                    contentDescription = null,
                    tint = Color.White,
                    modifier = Modifier.size(20.dp)
                )

            when (DisplayManager.size) {
                Size.Small -> {}
                Size.Medium, Size.Large -> {
                    if (DisplayManager.orientation == Orientation.Landscape) {
                        Spacer(Modifier.padding(start = 8.dp))
                        Text(
                            text = text,
                            style = MaterialTheme.typography.headlineSmall,
                            color = Color.White
                        )
                    }
                }
            }

            if(forward)
                Icon(
                    imageVector = Icons.AutoMirrored.Default.ArrowForward,
                    contentDescription = null,
                    tint = Color.White,
                    modifier = Modifier.size(20.dp)
                )
        }
    }
}
