package com.menagerie.bakers.view.util

import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.lazy.grid.GridCells
import androidx.compose.foundation.lazy.grid.LazyVerticalGrid
import androidx.compose.material3.Button
import androidx.compose.material3.Card
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.ArrowDropDown
import androidx.compose.material.icons.filled.PlayArrow
import androidx.compose.material3.FilterChip
import androidx.compose.material3.Icon
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.compose.ui.window.Dialog
import com.menagerie.bakers.model.TagType

@Composable
fun FilterCard(
    recipeTags: Map<String, TagType>,
    activeTags: List<String>,
    onDismissRequest : (List<String>) -> Unit,
) {

    var tagsFiltered: List<String> = activeTags
    Dialog(
        onDismissRequest = {
            onDismissRequest(activeTags)
        },
    ) {
        Card {
            Box {
                Column {
                    Row {
                        Button(onClick = { onDismissRequest(tagsFiltered) }) {
                            Text("Apply")
                        }
                        Spacer(modifier = Modifier.weight(1f))
                        Button(onClick = { onDismissRequest(listOf()) }) {
                            Text("Clear")
                        }
                    }
                    for (type in recipeTags.values.toSet().toList().sorted()) {
                        val list = recipeTags.filterValues { it == type }.keys.toList()
                            .sorted() // get list of all keys that have this value

                        var show by remember { mutableStateOf(false) }


                        Row {
                            Text(
                                "Cook By ".plus(type.name),
                                modifier = Modifier.clickable {
                                    show = !show
                                })
                            Icon(
                                imageVector = if (show) Icons.Default.PlayArrow else Icons.Default.ArrowDropDown,
                                "Open"
                            )
                        }

                        if (show) {
                            LazyVerticalGrid(
                                columns = GridCells.Adaptive(minSize = 110.dp)
                            ) {
                                items(list.size) { tag ->
                                    val active = activeTags.contains(list[tag])
                                    var selected by remember { mutableStateOf(active) }
                                    FilterChip(
                                        selected = selected,
                                        onClick = {
                                            selected = !selected
                                            tagsFiltered = if (selected) {
                                                tagsFiltered.plus(list[tag])
                                            } else {
                                                tagsFiltered.minus(list[tag])
                                            }
                                        },
                                        label = { Text(list[tag]) }
                                    )
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}