package com.menagerie.bakers.view.util

import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp

@Composable
fun InputFieldComponent(
    text: String,
    onChange: (String) -> Unit,
    modifier: Modifier = Modifier,
    singleLine: Boolean = true,
    label: String = "Some val",
    keyboardActions: KeyboardActions = KeyboardActions.Default,
) {
    OutlinedTextField(
        value = text,
        onValueChange = onChange,
        label = { Text(text = label) },
        modifier = modifier,
        singleLine = singleLine,
        keyboardActions = keyboardActions,
    )
}


@Composable
fun InputFieldState(
    value: String,
    label: String,
    modifier: Modifier,
    keyboardActions: KeyboardActions = KeyboardActions.Default,
    onVal: (String) -> Unit,
) {

    Column(
        modifier = Modifier
    ) {
        InputField(value, label, modifier, onVal, keyboardActions)
        Spacer(modifier = Modifier.padding(10.dp))
    }
}

@Composable
fun InputField(
    text: String,
    label: String,
    modifier: Modifier,
    onValChange: ((String) -> Unit)?,
    keyboardActions: KeyboardActions,
) {
    if (onValChange != null) {
        InputFieldComponent(
            text = text,
            onChange = onValChange,
            label = label,
            modifier = modifier,
            keyboardActions = keyboardActions,
        )
    }
}