package com.menagerie.bakers.view

import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material3.Checkbox
import androidx.compose.material3.HorizontalDivider
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import com.menagerie.bakers.Theme
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import com.menagerie.bakers.DisplayManager
import com.menagerie.bakers.Orientation
import com.menagerie.bakers.SortBy
import com.menagerie.bakers.model.TTT
import com.menagerie.bakers.view.util.PartiallyClickableText

@Composable
actual fun SettingsMenu(
    theme: Theme,
    onTheme: () -> Unit,
    animate: Boolean,
    onAnim: () -> Unit,
    helpUs: Boolean,
    onHelpToggle: (Boolean) -> Unit,
    ttt: TTT,
    onTTTToggle: () -> Unit,
    andOr: Boolean,
    onAndOr: () -> Unit,
    sortBy: SortBy,
    onSortToggle: () -> Unit,
    descending: Boolean,
    onDesc: () -> Unit,
    onRandom: () -> Unit,
    onBack: () -> Unit,
    onClear: () -> Unit,
    discreet: Boolean,
    onDiscreet: () -> Unit,
) {
    Box(
        modifier = Modifier.fillMaxSize()
    ) {
        Row {
            Column(modifier = Modifier.weight(1f)) {
                Row {
                    IconButton(
                        onClick = onBack,
                        content = {
                            Icon(
                                imageVector = Icons.AutoMirrored.Filled.ArrowBack,
                                contentDescription = "Back"
                            )
                        }
                    )
                }

                Row(modifier = Modifier.padding(16.dp)) {
                    Column(verticalArrangement = Arrangement.spacedBy(8.dp)) {
                        //Display Settings
                        Text(
                            text = "Display Settings",
                            style = MaterialTheme.typography.headlineSmall
                        )

                        PartiallyClickableText("Current Theme: ", theme.name, onTheme)

                        PartiallyClickableText(
                            "Play Animations: ",
                            if (animate) "Yes" else "No",
                            onAnim
                        )
                        HorizontalDivider()
                        //END Display

                        //Filter Stuff
                        Text(
                            text = "Filter Settings",
                            style = MaterialTheme.typography.headlineSmall
                        )

                        PartiallyClickableText(
                            "Filter Style:",
                            if (andOr) "OR" else "AND",
                            onAndOr
                        )

                        PartiallyClickableText(
                            "Sort By:",
                            sortBy.name,
                            onSortToggle,
                        )

                        PartiallyClickableText(
                            "Sort Order:",
                            if (descending) "DESC" else "ASC",
                            onDesc
                        )
                        //END Filter
                        HorizontalDivider()

                        //Beta Recipes
                        Text(
                            text = "Beta Recipe Settings",
                            style = MaterialTheme.typography.headlineSmall
                        )

                        PartiallyClickableText("Use extra discreet steps for recipes?",  if (discreet) "Yes" else "No", onDiscreet)

                        PartiallyClickableText("Show", ttt.name, onTTTToggle, "recipes or better")

                        Row {
                            Text(
                                text = "Help Us Improve! Show Only Recipes with Missing Info?",
                                modifier = Modifier.align(Alignment.CenterVertically)
                            )
                            Checkbox(
                                checked = helpUs,
                                onCheckedChange = onHelpToggle,
                                modifier = Modifier.align(Alignment.CenterVertically)
                            )
                        }
                        //END BETA

                        HorizontalDivider()

                        //DANGER
                        Text(
                            text = "DANGER ZONE",
                            style = MaterialTheme.typography.headlineSmall
                        )

                        PartiallyClickableText(
                            "Clear Settings?",
                            clickable = "CLEAR ALL SETTINGS",
                            onClick = onClear
                        )
                    }
                }
            }
            Column(modifier = Modifier.weight(if(DisplayManager.orientation == Orientation.Landscape) 4f else 2f)){ } //empty weight pusher
        }
    }
}